@@ -24,6 +24,8 @@
 #include "mate-wp-info.h"
 #include "mate-wp-item.h"
 #include "mate-wp-xml.h"
+#include "appearance-desktop.h"
+
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <string.h>
@@ -605,7 +607,7 @@ wp_props_wp_selected (GtkTreeSelection *selection,
   }
 }
 
-void
+static void
 wp_create_filechooser (AppearanceData *data)
 {
   const char *start_dir, *pictures = NULL;
@@ -613,7 +615,7 @@ wp_create_filechooser (AppearanceData *data)
 
   data->wp_filesel = GTK_FILE_CHOOSER (
                      gtk_file_chooser_dialog_new (_("Add Wallpaper"),
-                     GTK_WINDOW (appearance_capplet_get_widget (data, "appearance_window")),
+                     GET_WINDOW ("appearance_window"),
                      GTK_FILE_CHOOSER_ACTION_OPEN,
                      "gtk-cancel",
                      GTK_RESPONSE_CANCEL,
@@ -816,7 +818,7 @@ wp_update_preview (GtkFileChooser *chooser,
   if (uri)
   {
     GdkPixbuf *pixbuf = NULL;
-    const gchar *mime_type = NULL;
+    gchar *mime_type = NULL;
     GFile *file;
     GFileInfo *file_info;
 
@@ -829,15 +831,19 @@ wp_update_preview (GtkFileChooser *chooser,
 
     if (file_info != NULL)
     {
-      mime_type = g_file_info_get_content_type (file_info);
+      const gchar *content_type;
+
+      if ((content_type = g_file_info_get_content_type (file_info)) != NULL)
+        mime_type = g_content_type_get_mime_type (content_type);
       g_object_unref (file_info);
     }
 
     if (mime_type)
     {
       pixbuf = mate_desktop_thumbnail_factory_generate_thumbnail (data->thumb_factory,
-								   uri,
-								   mime_type);
+                                                                  uri,
+                                                                  mime_type);
+      g_free (mime_type);
     }
 
     if (pixbuf != NULL)
@@ -1328,8 +1334,6 @@ desktop_init (AppearanceData *data,
                     (GCallback) wp_scolor_changed, data);
 
   add_button = appearance_capplet_get_widget (data, "wp_add_button");
-  gtk_button_set_image (GTK_BUTTON (add_button),
-                        gtk_image_new_from_icon_name ("list-add", GTK_ICON_SIZE_BUTTON));
 
   g_signal_connect (add_button, "clicked",
                     (GCallback) wp_file_open_dialog, data);
@@ -1376,8 +1380,7 @@ desktop_shutdown (AppearanceData *data)
     data->screen_size_handler = 0;
   }
 
-  g_slist_foreach (data->wp_uris, (GFunc) g_free, NULL);
-  g_slist_free (data->wp_uris);
+  g_slist_free_full (data->wp_uris, g_free);
   if (data->wp_filesel)
   {
     g_object_ref_sink (data->wp_filesel);
