@@ -23,6 +23,6 @@
 #define THEME_INSTALLER_H
 
 void mate_theme_install (GFile *file, GtkWindow *parent);
-void mate_theme_installer_run (GtkWindow *parent, const gchar *filename);
+void mate_theme_installer_run (GtkWindow *parent);
 
 #endif /* THEME_INSTALLER_H */
