@@ -19,6 +19,7 @@ static char *
 wm_common_get_window_manager_property (Atom atom)
 {
   Atom utf8_string, type;
+  GdkDisplay *display;
   int result;
   char *retval;
   int format;
@@ -29,12 +30,13 @@ wm_common_get_window_manager_property (Atom atom)
   if (wm_window == None)
     return NULL;
 
-  utf8_string = XInternAtom (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "UTF8_STRING", False);
+  utf8_string = gdk_x11_get_xatom_by_name ("UTF8_STRING");
 
-  gdk_error_trap_push ();
+  display = gdk_display_get_default ();
+  gdk_x11_display_error_trap_push (display);
 
   val = NULL;
-  result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()),
+  result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY(display),
 		  	       wm_window,
 			       atom,
 			       0, G_MAXLONG,
@@ -42,7 +44,7 @@ wm_common_get_window_manager_property (Atom atom)
 			       &type, &format, &nitems,
 			       &bytes_after, (guchar **) &val);
 
-  if (gdk_error_trap_pop () || result != Success ||
+  if (gdk_x11_display_error_trap_pop (display) || result != Success ||
       type != utf8_string || format != 8 || nitems == 0 ||
       !g_utf8_validate (val, nitems, NULL))
     {
@@ -62,7 +64,7 @@ wm_common_get_window_manager_property (Atom atom)
 char*
 wm_common_get_current_window_manager (void)
 {
-  Atom atom = XInternAtom (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "_NET_WM_NAME", False);
+  Atom atom = gdk_x11_get_xatom_by_name ("_NET_WM_NAME");
   char *result;
 
   result = wm_common_get_window_manager_property (atom);
@@ -75,7 +77,7 @@ wm_common_get_current_window_manager (void)
 char**
 wm_common_get_current_keybindings (void)
 {
-  Atom keybindings_atom = XInternAtom (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "_MATE_WM_KEYBINDINGS", False);
+  Atom keybindings_atom = gdk_x11_get_xatom_by_name ("_MATE_WM_KEYBINDINGS");
   char *keybindings = wm_common_get_window_manager_property (keybindings_atom);
   char **results;
 
@@ -89,7 +91,7 @@ wm_common_get_current_keybindings (void)
     }
   else
     {
-      Atom wm_atom = XInternAtom (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "_NET_WM_NAME", False);
+      Atom wm_atom = gdk_x11_get_xatom_by_name ("_NET_WM_NAME");
       char *wm_name = wm_common_get_window_manager_property (wm_atom);
       char *to_copy[] = { NULL, NULL };
 
@@ -105,14 +107,16 @@ wm_common_get_current_keybindings (void)
 static void
 update_wm_window (void)
 {
+  GdkDisplay *display;
   Window *xwindow;
   Atom type;
   gint format;
   gulong nitems;
   gulong bytes_after;
 
-  XGetWindowProperty (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), GDK_ROOT_WINDOW (),
-		      XInternAtom (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "_NET_SUPPORTING_WM_CHECK", False),
+  display = gdk_display_get_default ();
+  XGetWindowProperty (GDK_DISPLAY_XDISPLAY(display), GDK_ROOT_WINDOW (),
+		      gdk_x11_get_xatom_by_name ("_NET_SUPPORTING_WM_CHECK"),
 		      0, G_MAXLONG, False, XA_WINDOW, &type, &format,
 		      &nitems, &bytes_after, (guchar **) &xwindow);
 
@@ -122,11 +126,11 @@ update_wm_window (void)
      return;
     }
 
-  gdk_error_trap_push ();
-  XSelectInput (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), *xwindow, StructureNotifyMask | PropertyChangeMask);
-  XSync (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), False);
+  gdk_x11_display_error_trap_push (display);
+  XSelectInput (GDK_DISPLAY_XDISPLAY(display), *xwindow, StructureNotifyMask | PropertyChangeMask);
+  XSync (GDK_DISPLAY_XDISPLAY(display), False);
 
-  if (gdk_error_trap_pop ())
+  if (gdk_x11_display_error_trap_pop (display))
     {
        XFree (xwindow);
        wm_window = None;
@@ -149,10 +153,10 @@ wm_window_event_filter (GdkXEvent *xev,
        wm_window != None && xevent->xany.window == wm_window) ||
       (xevent->type == PropertyNotify &&
        xevent->xany.window == GDK_ROOT_WINDOW () &&
-       xevent->xproperty.atom == (XInternAtom (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()),  "_NET_SUPPORTING_WM_CHECK", False))) ||
+       xevent->xproperty.atom == (gdk_x11_get_xatom_by_name ("_NET_SUPPORTING_WM_CHECK"))) ||
       (xevent->type == PropertyNotify &&
        wm_window != None && xevent->xany.window == wm_window &&
-       xevent->xproperty.atom == (XInternAtom (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "_NET_WM_NAME", False))))
+       xevent->xproperty.atom == (gdk_x11_get_xatom_by_name ("_NET_WM_NAME"))))
     {
       update_wm_window ();
       (* ncb_data->func) ((gpointer)wm_common_get_current_window_manager(),
