@@ -342,54 +342,10 @@ refresh_combo_box_icons(GtkIconTheme* theme, GtkComboBox* combo_box, GList* app_
 	}
 }
 
-static struct {
-	const char* name;
-	const char* icon;
-} icons[] = {
-	{"web_browser_image",  "web-browser"},
-	{"mail_reader_image",  "emblem-mail"},
-	{"media_player_image", "audio-x-generic"},
-	{"visual_image",       "zoom-best-fit"},
-	{"mobility_image",     "preferences-desktop-accessibility"},
-	{"messenger_image",    "instant-messaging"},
-	{"filemanager_image",  "file-manager"},
-	{"imageviewer_image",  "image-x-generic"},
-	{"video_image",        "video-x-generic"},
-	{"text_image",         "text-editor"},
-	{"terminal_image",     "terminal"},
-	{"document_image",     "application-pdf"},
-	{"word_image",         "x-office-document"},
-	{"spreadsheet_image",  "x-office-spreadsheet"},
-	{"calculator_image",   "accessories-calculator"},
-};
-
 /* Callback for icon theme change */
 static void
 theme_changed_cb(GtkIconTheme* theme, MateDACapplet* capplet)
 {
-	GObject* icon;
-	gint i;
-	gint scale_factor;
-	cairo_surface_t *surface;
-
-	scale_factor = gtk_widget_get_scale_factor (capplet->window);
-
-	for (i = 0; i < G_N_ELEMENTS(icons); i++)
-	{
-		icon = gtk_builder_get_object(capplet->builder, icons[i].name);
-
-		surface = gtk_icon_theme_load_surface (theme, icons[i].icon,
-		                                       32, scale_factor,
-		                                       NULL,
-		                                       GTK_ICON_LOOKUP_FORCE_SIZE,
-		                                       NULL);
-
-		gtk_image_set_from_surface (GTK_IMAGE(icon), surface);
-
-		if (surface)
-			cairo_surface_destroy (surface);
-	}
-
 	refresh_combo_box_icons(theme, GTK_COMBO_BOX(capplet->web_combo_box), capplet->web_browsers);
 	refresh_combo_box_icons(theme, GTK_COMBO_BOX(capplet->mail_combo_box), capplet->mail_readers);
 	refresh_combo_box_icons(theme, GTK_COMBO_BOX(capplet->media_combo_box), capplet->media_players);
@@ -630,23 +586,8 @@ show_dialog(MateDACapplet* capplet, const gchar* start_page)
 	#define get_widget(name) GTK_WIDGET(gtk_builder_get_object(builder, name))
 
 	GtkBuilder* builder;
-	guint builder_result;
 
-	capplet->builder = builder = gtk_builder_new ();
-
-	builder_result = gtk_builder_add_from_resource (builder, "/org/mate/mcc/da/mate-default-applications-properties.ui", NULL);
-
-	if (builder_result == 0)
-	{
-		GtkWidget* dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Could not load the main interface"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), _("Please make sure that the applet is properly installed"));
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-
-		gtk_dialog_run(GTK_DIALOG(dialog));
-
-		gtk_widget_destroy(dialog);
-		exit(EXIT_FAILURE);
-	}
+	capplet->builder = builder = gtk_builder_new_from_resource ("/org/mate/mcc/da/mate-default-applications-properties.ui");
 
 	capplet->window = get_widget("preferred_apps_dialog");
 
@@ -791,9 +732,10 @@ show_dialog(MateDACapplet* capplet, const gchar* start_page)
 
         GtkNotebook* nb = GTK_NOTEBOOK(get_widget("preferred_apps_notebook"));
         gtk_widget_add_events (GTK_WIDGET (nb), GDK_SCROLL_MASK);
-        g_signal_connect (GTK_WIDGET (nb), "scroll-event",
-                          G_CALLBACK (capplet_dialog_page_scroll_event_cb),
-                          GTK_WINDOW (capplet->window));
+        g_signal_connect (GTK_WIDGET (nb),
+                          "scroll-event",
+                          G_CALLBACK (capplet_notebook_scroll_event_cb),
+                          NULL);
 
 	if (start_page != NULL)
 	{
@@ -847,6 +789,7 @@ main(int argc, char** argv)
 	g_option_context_add_main_entries(context, option_entries, GETTEXT_PACKAGE);
 
 	capplet_init(context, &argc, &argv);
+	g_option_context_free(context);
 
 	MateDACapplet* capplet = g_new0(MateDACapplet, 1);
 
