@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2019 MATE Developers
- * Copyright (C) 2018, 2019 zhuyaliang https://github.com/zhuyaliang/
- * Copyright (C) 2010-2018 The GNOME Project
  * Copyright (C) 2010 Intel, Inc
+ * Copyright (C) 2010-2018 The GNOME Project
+ * Copyright (C) 2018, 2019 zhuyaliang https://github.com/zhuyaliang/
+ * Copyright (C) 2019-2021 MATE Developers
  *
  * Portions from Ubiquity, Copyright (C) 2009 Canonical Ltd.
  * Written by Evan Dandrea <evand@ubuntu.com>
@@ -57,8 +57,8 @@ static TimezoneMapOffset color_codes[] =
     {-9.0, 128, 0, 0, 255 },
     {-8.0, 170, 0, 0, 255 },
     {-7.0, 212, 0, 0, 255 },
-    {-6.0, 255, 0, 1, 255 }, // north
-    {-6.0, 255, 0, 0, 255 }, // south
+    {-6.0, 255, 0, 1, 255 }, /* north */
+    {-6.0, 255, 0, 0, 255 }, /* south */
     {-5.0, 255, 42, 42, 255 },
     {-4.5, 192, 255, 0, 255 },
     {-4.0, 255, 85, 85, 255 },
