@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2019 MATE Developers
- * Copyright (C) 2018, 2019 zhuyaliang https://github.com/zhuyaliang/
- * Copyright (C) 2010-2018 The GNOME Project
  * Copyright (C) 2010 Intel, Inc
+ * Copyright (C) 2010-2018 The GNOME Project
+ * Copyright (C) 2018, 2019 zhuyaliang https://github.com/zhuyaliang/
+ * Copyright (C) 2019-2021 MATE Developers
  *
  * Portions from Ubiquity, Copyright (C) 2009 Canonical Ltd.
  * Written by Evan Dandrea <evand@ubuntu.com>
@@ -36,10 +36,9 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libintl.h>
 #include <glib/gi18n.h>
 
-#include <libmate-desktop/mate-languages.h>
+#include <libmate-desktop/mate-languages.h> /* mate_get_country_from_code */
 
 #define DEFAULT_TZ "Europe/London"
 #define BACKFILE   TIMPZONEDIR "backward"
@@ -297,7 +296,16 @@ update_timezone (TimezoneMap *map)
     current_location = timezone_map_get_location (TIMEZONEMAP (map));
     city_country = translated_city_name (current_location);
 
+#if GLIB_CHECK_VERSION (2, 68, 0)
+    if ((timezone =  g_time_zone_new_identifier (current_location->zone)) == NULL) {
+        timezone_map_set_bubble_text (TIMEZONEMAP (map),
+                                      _("Error: timezone identifier cannot be parsed or loaded"));
+        return;
+    }
+#else
     timezone = g_time_zone_new (current_location->zone);
+#endif
+
     current_date = g_date_time_new_now_local ();
     date = g_date_time_to_timezone (current_date, timezone);
 
@@ -363,11 +371,12 @@ static void
 CreateCityList(TimeAdmin *ta)
 {
     TzDB *db;
-    ta->CityListStore = gtk_list_store_new (CITY_NUM_COLS,G_TYPE_STRING,G_TYPE_STRING);
-    db = tz_load_db ();
-    g_ptr_array_foreach (db->locations, (GFunc) LoadCities, ta->CityListStore);
 
-    TimeZoneDateBaseFree(db);
+    ta->CityListStore = gtk_list_store_new (CITY_NUM_COLS, G_TYPE_STRING, G_TYPE_STRING);
+    if ((db = tz_load_db ()) != NULL) {
+        g_ptr_array_foreach (db->locations, (GFunc) LoadCities, ta->CityListStore);
+        TimeZoneDateBaseFree (db);
+    }
 }
 
 static GtkWidget*
