@@ -1,5 +1,6 @@
 /* mate-metacity-support.c
  * Copyright (C) 2014 Stefano Karapetsas
+ * Copyright (C) 2019-2021 MATE Developers
  *
  * Written by: Stefano Karapetsas <stefano@karapetsas.com>
  *
@@ -23,11 +24,13 @@
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 
+#include "mate-metacity-support.h"
+
 #define METACITY_SCHEMA "org.gnome.metacity"
 #define COMPOSITING_KEY "compositing-manager"
 
 void
-mate_metacity_config_tool ()
+mate_metacity_config_tool (void)
 {
     GSettings *settings;
     GtkDialog *dialog;
