@@ -40,22 +40,39 @@
 #include "capplet-util.h"
 
 #define MARCO_SCHEMA "org.mate.Marco.general"
-#define MARCO_THEME_KEY "theme"
-#define MARCO_FONT_KEY  "titlebar-font"
+#define INTERFACE_SCHEMA "org.mate.interface"
+
+#define GTK_BUTTON_LAYOUT_KEY "gtk-decoration-layout"
+
+#define MARCO_CENTER_NEW_WINDOWS_KEY "center-new-windows"
+#define MARCO_ALLOW_TILING_KEY "allow-tiling"
+#define MARCO_ALLOW_TOP_TILING_KEY "allow-top-tiling"
+#define MARCO_SHOW_TAB_BORDER_KEY "show-tab-border"
+#define MARCO_BUTTON_LAYOUT_KEY "button-layout"
+#define MARCO_DOUBLE_CLICK_TITLEBAR_KEY "action-double-click-titlebar"
 #define MARCO_FOCUS_KEY "focus-mode"
-#define MARCO_USE_SYSTEM_FONT_KEY "titlebar-uses-system-font"
 #define MARCO_AUTORAISE_KEY "auto-raise"
 #define MARCO_AUTORAISE_DELAY_KEY "auto-raise-delay"
 #define MARCO_MOUSE_MODIFIER_KEY "mouse-button-modifier"
-#define MARCO_DOUBLE_CLICK_TITLEBAR_KEY "action-double-click-titlebar"
+
 #define MARCO_COMPOSITING_MANAGER_KEY "compositing-manager"
 #define MARCO_COMPOSITING_FAST_ALT_TAB_KEY "compositing-fast-alt-tab"
-#define MARCO_ALLOW_TILING_KEY "allow-tiling"
-#define MARCO_CENTER_NEW_WINDOWS_KEY "center-new-windows"
-#define MARCO_BUTTON_LAYOUT_KEY "button-layout"
 
-#define MARCO_BUTTON_LAYOUT_RIGHT "menu:minimize,maximize,close"
-#define MARCO_BUTTON_LAYOUT_LEFT "close,minimize,maximize:"
+enum
+{
+    MARCO_BUTTON_LAYOUT_RIGHT_WITH_MENU = 0,
+    MARCO_BUTTON_LAYOUT_LEFT_WITH_MENU,
+    MARCO_BUTTON_LAYOUT_RIGHT,
+    MARCO_BUTTON_LAYOUT_LEFT,
+    MARCO_BUTTON_LAYOUT_COUNT
+};
+
+static const char *button_layout [MARCO_BUTTON_LAYOUT_COUNT] = {
+   [MARCO_BUTTON_LAYOUT_RIGHT_WITH_MENU] = "menu:minimize,maximize,close",
+   [MARCO_BUTTON_LAYOUT_LEFT_WITH_MENU]  = "close,minimize,maximize:menu",
+   [MARCO_BUTTON_LAYOUT_RIGHT]           = ":minimize,maximize,close",
+   [MARCO_BUTTON_LAYOUT_LEFT]            = "close,minimize,maximize:"
+};
 
 /* keep following enums in sync with marco */
 enum
@@ -69,6 +86,7 @@ enum
     ACTION_TITLEBAR_LOWER,
     ACTION_TITLEBAR_MENU
 };
+
 enum
 {
     FOCUS_MODE_CLICK,
@@ -85,20 +103,29 @@ typedef struct
 } MouseClickModifier;
 
 static GtkWidget *dialog_win;
-static GtkWidget *compositing_checkbutton;
+
+/* Behaviour */
+static GtkWidget *show_tab_border_checkbutton;
 static GtkWidget *compositing_fast_alt_tab_checkbutton;
-static GtkWidget *allow_tiling_checkbutton;
-static GtkWidget *center_new_windows_checkbutton;
+static GtkWidget *double_click_titlebar_optionmenu;
 static GtkWidget *focus_mode_checkbutton;
 static GtkWidget *focus_mode_mouse_checkbutton;
 static GtkWidget *autoraise_checkbutton;
-static GtkWidget *autoraise_delay_slider;
+static GtkWidget *autoraise_delay_spinbutton;
 static GtkWidget *autoraise_delay_hbox;
-static GtkWidget *double_click_titlebar_optionmenu;
-static GtkWidget *titlebar_layout_optionmenu;
 static GtkWidget *alt_click_vbox;
 
+/* Placement */
+static GtkWidget *center_new_windows_checkbutton;
+static GtkWidget *enable_tiling_checkbutton;
+static GtkWidget *allow_top_tiling_checkbutton;
+static GtkWidget *titlebar_layout_optionmenu;
+
+/* Compositing Manager */
+static GtkWidget *compositing_checkbutton;
+
 static GSettings *marco_settings;
+static GSettings *interface_settings;
 
 static MouseClickModifier *mouse_modifiers = NULL;
 static int n_mouse_modifiers = 0;
@@ -106,25 +133,19 @@ static int n_mouse_modifiers = 0;
 static void reload_mouse_modifiers (void);
 
 static void
-update_sensitivity ()
+update_sensitivity (void)
 {
-    gchar *str;
-
-    gtk_widget_set_sensitive (GTK_WIDGET (compositing_fast_alt_tab_checkbutton),
+    gtk_widget_set_sensitive (compositing_fast_alt_tab_checkbutton,
                               g_settings_get_boolean (marco_settings, MARCO_COMPOSITING_MANAGER_KEY));
-    gtk_widget_set_sensitive (GTK_WIDGET (focus_mode_mouse_checkbutton),
+    gtk_widget_set_sensitive (allow_top_tiling_checkbutton,
+                              g_settings_get_boolean (marco_settings, MARCO_ALLOW_TILING_KEY));
+    gtk_widget_set_sensitive (focus_mode_mouse_checkbutton,
                               g_settings_get_enum (marco_settings, MARCO_FOCUS_KEY) != FOCUS_MODE_CLICK);
-    gtk_widget_set_sensitive (GTK_WIDGET (autoraise_checkbutton),
+    gtk_widget_set_sensitive (autoraise_delay_hbox,
                               g_settings_get_enum (marco_settings, MARCO_FOCUS_KEY) != FOCUS_MODE_CLICK);
-    gtk_widget_set_sensitive (GTK_WIDGET (autoraise_delay_hbox),
+    gtk_widget_set_sensitive (autoraise_delay_spinbutton,
                               g_settings_get_enum (marco_settings, MARCO_FOCUS_KEY) != FOCUS_MODE_CLICK &&
                               g_settings_get_boolean (marco_settings, MARCO_AUTORAISE_KEY));
-
-    str = g_settings_get_string (marco_settings, MARCO_BUTTON_LAYOUT_KEY);
-    gtk_widget_set_sensitive (GTK_WIDGET (titlebar_layout_optionmenu),
-                              g_strcmp0 (str, MARCO_BUTTON_LAYOUT_LEFT) == 0 ||
-                              g_strcmp0 (str, MARCO_BUTTON_LAYOUT_RIGHT) == 0);
-    g_free (str);
 }
 
 static void
@@ -170,12 +191,12 @@ mouse_focus_changed_callback (GSettings *settings,
 }
 
 static void
-autoraise_delay_value_changed_callback (GtkWidget *slider,
-                                        void      *data)
+autoraise_delay_spinbutton_value_callback (GtkWidget *spinbutton,
+                                           void      *data)
 {
     g_settings_set_int (marco_settings,
                         MARCO_AUTORAISE_DELAY_KEY,
-                        gtk_range_get_value (GTK_RANGE (slider)) * 1000);
+                        gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbutton)) * 1000);
 }
 
 static void
@@ -186,20 +207,50 @@ double_click_titlebar_changed_callback (GtkWidget *optionmenu,
                          gtk_combo_box_get_active (GTK_COMBO_BOX (optionmenu)));
 }
 
+static gchar *custom_titlebar_button_layout = NULL;
+
 static void
-titlebar_layout_changed_callback (GtkWidget *optionmenu,
-                                  void      *data)
+titlebar_layout_changed_callback (GtkWidget *optionmenu)
 {
     gint value = gtk_combo_box_get_active (GTK_COMBO_BOX (optionmenu));
 
-    if (value == 0) {
-        g_settings_set_string (marco_settings, MARCO_BUTTON_LAYOUT_KEY, MARCO_BUTTON_LAYOUT_RIGHT);
+    if (value < MARCO_BUTTON_LAYOUT_COUNT) {
+        g_settings_set_string (marco_settings, MARCO_BUTTON_LAYOUT_KEY, button_layout[value]);
+        g_settings_set_string (interface_settings, GTK_BUTTON_LAYOUT_KEY, button_layout[value]);
     }
-    else if (value == 1) {
-        g_settings_set_string (marco_settings, MARCO_BUTTON_LAYOUT_KEY, MARCO_BUTTON_LAYOUT_LEFT);
+    /* Custom Layout */
+    else if ((value == MARCO_BUTTON_LAYOUT_COUNT) && custom_titlebar_button_layout) {
+        g_settings_set_string (marco_settings, MARCO_BUTTON_LAYOUT_KEY, custom_titlebar_button_layout);
+        g_settings_set_string (interface_settings, GTK_BUTTON_LAYOUT_KEY, custom_titlebar_button_layout);
     }
 }
 
+static void
+set_titlebar_button_layout(void)
+{
+    gchar    *str;
+    gboolean  found = FALSE;
+
+    str = g_settings_get_string (marco_settings, MARCO_BUTTON_LAYOUT_KEY);
+    for (gint i = 0; i < MARCO_BUTTON_LAYOUT_COUNT; i++) {
+        if (g_strcmp0 (str, button_layout[i]) == 0) {
+            gtk_combo_box_set_active (GTK_COMBO_BOX (titlebar_layout_optionmenu), i);
+            found = TRUE;
+            break;
+        }
+    }
+    /* A custom value is found in MARCO_BUTTON_LAYOUT_KEY
+     * (maybe the user changed the value in dconf-editor) */
+    if (!found) {
+        g_free (custom_titlebar_button_layout);
+        custom_titlebar_button_layout = g_strdup(str);
+        gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (titlebar_layout_optionmenu), _("Custom"));
+        gtk_combo_box_set_active (GTK_COMBO_BOX (titlebar_layout_optionmenu), 4);
+    }
+
+    g_free(str);
+}
+
 static void
 alt_click_radio_toggled_callback (GtkWidget *radio,
                                   void      *data)
@@ -218,7 +269,7 @@ alt_click_radio_toggled_callback (GtkWidget *radio,
 }
 
 static void
-set_alt_click_value ()
+set_alt_click_value (void)
 {
     gboolean match_found = FALSE;
     gchar *mouse_move_modifier;
@@ -249,7 +300,7 @@ set_alt_click_value ()
 }
 
 static void
-wm_unsupported ()
+wm_unsupported (void)
 {
     GtkWidget *no_tool_dialog;
 
@@ -292,30 +343,12 @@ response_cb (GtkWidget *dialog_win,
     }
 }
 
-GtkWidget*
-title_label_new (const char* title)
-{
-    GtkWidget *widget;
-    gchar *str;
-
-    str = g_strdup_printf ("<b>%s</b>", _(title));
-    widget = gtk_label_new (str);
-    g_free (str);
-
-    gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
-    gtk_label_set_xalign (GTK_LABEL (widget), 0.0);
-    gtk_label_set_yalign (GTK_LABEL (widget), 0.0);
-
-    return widget;
-}
-
 int
 main (int argc, char **argv)
 {
-    GError     *error = NULL;
     GtkBuilder *builder;
     GdkScreen  *screen;
-    gchar      *str;
+    GtkWidget  *nb;
     const char *current_wm;
     int i;
 
@@ -334,21 +367,12 @@ main (int argc, char **argv)
         return 1;
     }
 
-    marco_settings = g_settings_new (MARCO_SCHEMA);
-
-    builder = gtk_builder_new ();
-    if (gtk_builder_add_from_resource (builder, "/org/mate/mcc/windows/window-properties.ui", &error) == 0) {
-        g_warning ("Could not load UI: %s", error->message);
-        g_error_free (error);
-        g_object_unref (marco_settings);
-        g_object_unref (builder);
-        return -1;
-    }
+    builder = gtk_builder_new_from_resource ("/org/mate/mcc/windows/window-properties.ui");
 
     gtk_builder_add_callback_symbols (builder,
                                       "on_dialog_win_response",                        G_CALLBACK (response_cb),
-                                      "on_autoraise_delay_slider_value_changed",       G_CALLBACK (autoraise_delay_value_changed_callback),
                                       "on_double_click_titlebar_optionmenu_changed",   G_CALLBACK (double_click_titlebar_changed_callback),
+                                      "on_autoraise_delay_spinbutton_value_changed",   G_CALLBACK (autoraise_delay_spinbutton_value_callback),
                                       "on_titlebar_layout_optionmenu_changed",         G_CALLBACK (titlebar_layout_changed_callback),
                                       "on_focus_mode_checkbutton_toggled",             G_CALLBACK (mouse_focus_toggled_callback),
                                       "on_focus_mode_mouse_checkbutton_toggled",       G_CALLBACK (mouse_focus_toggled_callback),
@@ -356,58 +380,63 @@ main (int argc, char **argv)
 
     gtk_builder_connect_signals (builder, NULL);
 
+    #define GET_WIDGET(x) GTK_WIDGET(gtk_builder_get_object(builder, x))
+
     /* Window */
-    dialog_win = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_win"));
+    dialog_win = GET_WIDGET ("dialog_win");
 
-    /* Compositing manager */
-    compositing_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "compositing_checkbutton"));
-    compositing_fast_alt_tab_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "compositing_fast_alt_tab_checkbutton"));
+    /* Notebook */
+    nb = GET_WIDGET ("nb");
+    gtk_widget_add_events (nb, GDK_SCROLL_MASK);
+    g_signal_connect (nb,
+                      "scroll-event",
+                      G_CALLBACK (capplet_notebook_scroll_event_cb),
+                      NULL);
 
-    /* Titlebar buttons */
-    titlebar_layout_optionmenu = GTK_WIDGET (gtk_builder_get_object (builder, "titlebar_layout_optionmenu"));
+    /* Behaviour */
+    show_tab_border_checkbutton = GET_WIDGET ("show_tab_border_checkbutton");
+    compositing_fast_alt_tab_checkbutton = GET_WIDGET ("compositing_fast_alt_tab_checkbutton");
+    double_click_titlebar_optionmenu = GET_WIDGET ("double_click_titlebar_optionmenu");
+    focus_mode_checkbutton = GET_WIDGET ("focus_mode_checkbutton");
+    focus_mode_mouse_checkbutton = GET_WIDGET ("focus_mode_mouse_checkbutton");
+    autoraise_delay_hbox = GET_WIDGET ("autoraise_delay_hbox");
+    autoraise_checkbutton = GET_WIDGET ("autoraise_checkbutton");
+    autoraise_delay_spinbutton = GET_WIDGET ("autoraise_delay_spinbutton");
+    alt_click_vbox = GET_WIDGET ("alt_click_vbox");
 
-    /* New Windows */
-    center_new_windows_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "center_new_windows_checkbutton"));
-
-    /* Window Snapping */
-    allow_tiling_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "allow_tiling_checkbutton"));
-
-    /* Window Selection */
-    focus_mode_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "focus_mode_checkbutton"));
-    focus_mode_mouse_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "focus_mode_mouse_checkbutton"));
-    autoraise_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "autoraise_checkbutton"));
-    autoraise_delay_hbox = GTK_WIDGET (gtk_builder_get_object (builder, "autoraise_delay_hbox"));
-    autoraise_delay_slider = GTK_WIDGET (gtk_builder_get_object (builder, "autoraise_delay_slider"));
-
-    /* Titlebar Action */
-    double_click_titlebar_optionmenu = GTK_WIDGET (gtk_builder_get_object (builder, "double_click_titlebar_optionmenu"));
-
-    /* Movement Key */
-    alt_click_vbox = GTK_WIDGET (gtk_builder_get_object (builder, "alt_click_vbox"));
+    /* Placement */
+    center_new_windows_checkbutton = GET_WIDGET ("center_new_windows_checkbutton");
+    enable_tiling_checkbutton = GET_WIDGET ("enable_tiling_checkbutton");
+    allow_top_tiling_checkbutton = GET_WIDGET ("allow_top_tiling_checkbutton");
+    titlebar_layout_optionmenu = GET_WIDGET ("titlebar_layout_optionmenu");
 
+    /* Compositing Manager */
+    compositing_checkbutton = GET_WIDGET ("compositing_checkbutton");
 
     g_object_unref (builder);
 
+    #undef GET_WIDGET
+
+    /* Load settings */
+    marco_settings = g_settings_new (MARCO_SCHEMA);
+    interface_settings = g_settings_new (INTERFACE_SCHEMA);
 
     reload_mouse_modifiers ();
 
-    str = g_settings_get_string (marco_settings, MARCO_BUTTON_LAYOUT_KEY);
-    gtk_combo_box_set_active (GTK_COMBO_BOX (titlebar_layout_optionmenu),
-                              g_strcmp0 (str, MARCO_BUTTON_LAYOUT_RIGHT) == 0 ? 0 : 1);
-    g_free (str);
-
     gtk_combo_box_set_active (GTK_COMBO_BOX (double_click_titlebar_optionmenu),
                               g_settings_get_enum (marco_settings, MARCO_DOUBLE_CLICK_TITLEBAR_KEY));
 
+    set_titlebar_button_layout ();
+
     set_alt_click_value ();
-    gtk_range_set_value (GTK_RANGE (autoraise_delay_slider),
-                         g_settings_get_int (marco_settings, MARCO_AUTORAISE_DELAY_KEY) / 1000.0);
+
     gtk_combo_box_set_active (GTK_COMBO_BOX (double_click_titlebar_optionmenu),
                               g_settings_get_enum (marco_settings, MARCO_DOUBLE_CLICK_TITLEBAR_KEY));
 
+    /* Behaviour */
     g_settings_bind (marco_settings,
-                     MARCO_COMPOSITING_MANAGER_KEY,
-                     compositing_checkbutton,
+                     MARCO_SHOW_TAB_BORDER_KEY,
+                     show_tab_border_checkbutton,
                      "active",
                      G_SETTINGS_BIND_DEFAULT);
 
@@ -415,17 +444,31 @@ main (int argc, char **argv)
                      MARCO_COMPOSITING_FAST_ALT_TAB_KEY,
                      compositing_fast_alt_tab_checkbutton,
                      "active",
-                     G_SETTINGS_BIND_DEFAULT);
+                     G_SETTINGS_BIND_DEFAULT | G_SETTINGS_BIND_INVERT_BOOLEAN);
 
+    /* Placement */
     g_settings_bind (marco_settings,
-                     MARCO_ALLOW_TILING_KEY,
-                     allow_tiling_checkbutton,
+                     MARCO_CENTER_NEW_WINDOWS_KEY,
+                     center_new_windows_checkbutton,
                      "active",
                      G_SETTINGS_BIND_DEFAULT);
 
     g_settings_bind (marco_settings,
-                     MARCO_CENTER_NEW_WINDOWS_KEY,
-                     center_new_windows_checkbutton,
+                     MARCO_ALLOW_TILING_KEY,
+                     enable_tiling_checkbutton,
+                     "active",
+                     G_SETTINGS_BIND_DEFAULT);
+
+    g_settings_bind (marco_settings,
+                     MARCO_ALLOW_TOP_TILING_KEY,
+                     allow_top_tiling_checkbutton,
+                     "active",
+                     G_SETTINGS_BIND_DEFAULT);
+
+    /* Compositing Manager */
+    g_settings_bind (marco_settings,
+                     MARCO_COMPOSITING_MANAGER_KEY,
+                     compositing_checkbutton,
                      "active",
                      G_SETTINGS_BIND_DEFAULT);
 
@@ -438,7 +481,6 @@ main (int argc, char **argv)
                      "active",
                      G_SETTINGS_BIND_DEFAULT);
 
-
     g_signal_connect (G_OBJECT (dialog_win), "destroy",
                       G_CALLBACK (gtk_main_quit), NULL);
 
@@ -467,6 +509,9 @@ main (int argc, char **argv)
     gtk_main ();
 
     g_object_unref (marco_settings);
+    g_object_unref (interface_settings);
+
+    g_free (custom_titlebar_button_layout);
 
     return 0;
 }
