@@ -45,6 +45,7 @@
 #define FONT_VIEW_APPLICATION(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_VIEW_TYPE_APPLICATION, FontViewApplication))
 
+static GType font_view_application_get_type (void);
 typedef struct {
     GtkApplication parent;
 
@@ -95,7 +96,6 @@ static const gchar *app_menu =
     "  <menu>"
     "</interface>";
 
-#define VIEW_ITEM_WIDTH 140
 #define VIEW_ITEM_WRAP_WIDTH 128
 #define VIEW_COLUMN_SPACING 36
 #define VIEW_MARGIN 16
@@ -945,9 +945,11 @@ main (int argc,
     GApplication *app;
     gint retval;
 
+#ifdef ENABLE_NLS
     bindtextdomain (GETTEXT_PACKAGE, MATELOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
+#endif /* ENABLE_NLS */
 
     app = font_view_application_new ();
     retval = g_application_run (app, argc, argv);
